#ifndef WX_PRECOMP
    #include "wx/wx.h"
#else
#include <wx/wxprec.h>
#endif

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "wx/busyinfo.h"
#include "wx/socket.h"
#include <wx/file.h>
#include <wx/process.h>
#include "SciWebLabConfig.h"

#if wxUSE_IPV6
    typedef wxIPV6address IPaddress;
#else
    typedef wxIPV4address IPaddress;
#endif

// --------------------------------------------------------------------------
// resources
// --------------------------------------------------------------------------

// the application icon
#if !defined(__WXMSW__) && !defined(__WXPM__)
    #include "sample.xpm"
#endif

#define __PRESTATAIRE_SCILAB__

#include "call_scilab.h" 
#include "ScilabThread.h"
#include <fstream>
#include <map>

using namespace std;

// --------------------------------------------------------------------------
// classes
// --------------------------------------------------------------------------

// Define a new application type
class MyApp : public wxApp
{
public:
  virtual bool OnInit();
};

// Define a new frame type: this is going to be our main frame
class MyFrame : public wxFrame
{
public:
  MyFrame();
  ~MyFrame();

  // event handlers (these functions should _not_ be virtual)
  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnServerEvent(wxSocketEvent& event);
  void OnSocketEvent(wxSocketEvent& event);
  void FinRequete(wxProcessEvent& event);


  // convenience functions
  void UpdateStatusBar();
  void GestionRequete(wxSocketBase *sock,char *buf);

private:
  wxSocketServer *m_server;
  wxTextCtrl     *m_text;
  wxMenu         *m_menuFile;
  wxMenuBar      *m_menuBar;
  bool            m_busy;
  int             m_numClients;
  int				nbAppel;
  wxProcess		 *wp;
std::map<int,ScilabThread *> listeRequete;			// liste nom de la balise avec profondeur


  // any class wishing to process wxWidgets events must use this macro
  DECLARE_EVENT_TABLE()
};


// --------------------------------------------------------------------------
// constants
// --------------------------------------------------------------------------

// IDs for the controls and the menu commands
enum
{
  // menu items
  SERVER_UDPTEST = 10,
  SERVER_WAITFORACCEPT,
  SERVER_QUIT = wxID_EXIT,
  SERVER_ABOUT = wxID_ABOUT,

  // id for sockets
  SERVER_ID = 100,
  SOCKET_ID,
  SCILAB_ID
};

// --------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// --------------------------------------------------------------------------

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(SERVER_QUIT,  MyFrame::OnQuit)
  EVT_MENU(SERVER_ABOUT, MyFrame::OnAbout)
 // EVT_MENU(SERVER_UDPTEST, MyFrame::OnUDPTest)
  EVT_SOCKET(SERVER_ID,  MyFrame::OnServerEvent)
  EVT_SOCKET(SOCKET_ID,  MyFrame::OnSocketEvent)
  EVT_END_PROCESS(SCILAB_ID,MyFrame::FinRequete)
END_EVENT_TABLE()

IMPLEMENT_APP(MyApp)


// ==========================================================================
// implementation
// ==========================================================================

// --------------------------------------------------------------------------
// the application class
// --------------------------------------------------------------------------

bool MyApp::OnInit()
{
    if ( !wxApp::OnInit() )
        return false;

char *sciPath="/usr/share/scilab-5.5.0/share/scilab";
char *sciStart="/usr/share/scilab";
#ifdef _MSC_VER
if ( StartScilab(NULL,NULL,NULL) == FALSE )
#else
if ( StartScilab(sciPath,NULL,NULL) == FALSE )
#endif
  {
   fprintf(stderr,"Error while calling StartScilab\n");
   return -1;
  }
 //DisableInteractiveMode();
  if ( !wxApp::OnInit() )
      return false;

  // Create the main application window
  MyFrame *frame = new MyFrame();

  // Show it
  frame->Show(true);

  // Success
  return true;
}

// --------------------------------------------------------------------------
// main frame
// --------------------------------------------------------------------------

// frame constructor

MyFrame::MyFrame() : wxFrame((wxFrame *)NULL, wxID_ANY,
                             _("wxSocket demo: Server"),
                             wxDefaultPosition, wxSize(300, 200))
{
ifstream	fConfig("config.txt");
int			indPort;
string		chaine;

if (!fConfig.is_open())
	{
	wxMessageBox("Fichier de configuration absent");
	wxExit();
	}
fConfig>>chaine>>indPort;
fConfig.close();
  // Give the frame an icon
  //SetIcon(wxICON(sample));
nbAppel=0;
wp = new wxProcess(this,SCILAB_ID);
  // Make menus
  m_menuFile = new wxMenu();
  m_menuFile->Append(SERVER_WAITFORACCEPT, "&Wait for connection\tCtrl-W");
  m_menuFile->Append(SERVER_UDPTEST, "&UDP test\tCtrl-U");
  m_menuFile->AppendSeparator();
  m_menuFile->Append(SERVER_ABOUT, _("&About...\tCtrl-A"), _("Show about dialog"));
  m_menuFile->AppendSeparator();
  m_menuFile->Append(SERVER_QUIT, _("E&xit\tAlt-X"), _("Quit server"));

  // Append menus to the menubar
  m_menuBar = new wxMenuBar();
  m_menuBar->Append(m_menuFile, _("&File"));
  SetMenuBar(m_menuBar);

#if wxUSE_STATUSBAR
  // Status bar
  CreateStatusBar(2);
#endif // wxUSE_STATUSBAR

  // Make a textctrl for logging
  m_text  = new wxTextCtrl(this, wxID_ANY,
                           _("Welcome to wxSocket demo: Server\n"),
                           wxDefaultPosition, wxDefaultSize,
                           wxTE_MULTILINE | wxTE_READONLY);
  delete wxLog::SetActiveTarget(new wxLogTextCtrl(m_text));

  // Create the address - defaults to localhost:0 initially
  IPaddress addr;
  addr.Service(indPort);

  wxLogMessage("Creating server at %s:%u", addr.IPAddress(), addr.Service());

  // Create the socket
  m_server = new wxSocketServer(addr);

  // We use IsOk() here to see if the server is really listening
  if (! m_server->IsOk())
  {
    wxMessageBox("Could not listen at the specified port !");
    wxExit();

  }

  IPaddress addrReal;
  if ( !m_server->GetLocal(addrReal) )
  {
    wxLogMessage("ERROR: couldn't get the address we bound to");
  }
  else
  {
    wxLogMessage("Server listening at %s:%u",
                 addrReal.IPAddress(), addrReal.Service());
  }

  // Setup the event handler and subscribe to connection events
  m_server->SetEventHandler(*this, SERVER_ID);
  m_server->SetNotify(wxSOCKET_CONNECTION_FLAG);
  m_server->Notify(true);

  m_busy = false;
  m_numClients = 0;
  UpdateStatusBar();
}

MyFrame::~MyFrame()
{
  // No delayed deletion here, as the frame is dying anyway
  delete m_server;
}

// event handlers

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
  // true is to force the frame to close
  Close(true);
}

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxMessageBox(_("wxSocket demo: Server\n(c) 1999 Guillermo Rodriguez Garcia\n"),
               _("About Server"),
               wxOK | wxICON_INFORMATION, this);
}




void MyFrame::OnServerEvent(wxSocketEvent& event)
{
  wxString s = _("OnServerEvent: ");
  wxSocketBase *sock;

  switch(event.GetSocketEvent())
  {
    case wxSOCKET_CONNECTION : s.Append(_("wxSOCKET_CONNECTION\n")); break;
    default                  : s.Append(_("Unexpected event !\n")); break;
  }

  m_text->AppendText(s);

  // Accept new connection if there is one in the pending
  // connections queue, else exit. We use Accept(false) for
  // non-blocking accept (although if we got here, there
  // should ALWAYS be a pending connection).

  sock = m_server->Accept(false);

  if (sock)
  {
    IPaddress addr;
    if ( !sock->GetPeer(addr) )
    {
      wxLogMessage("New connection from unknown client accepted.");
    }
    else
    {
      wxLogMessage("New client connection from %s:%u accepted",
                   addr.IPAddress(), addr.Service());
    }
  }
  else
  {
    wxLogMessage("Error: couldn't accept a new connection");
    return;
  }

  sock->SetEventHandler(*this, SOCKET_ID);
  sock->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG);
  sock->Notify(true);

  m_numClients++;
  UpdateStatusBar();
}

void MyFrame::OnSocketEvent(wxSocketEvent& event)
{
 wxString s = _("OnSocketEvent: ");
 wxSocketBase *sock = event.GetSocket();

  // Information affiche dans la fentre de log
 switch(event.GetSocketEvent())
	{
	case wxSOCKET_INPUT : s.Append(_("wxSOCKET_INPUT\n")); break;
	case wxSOCKET_LOST  : s.Append(_("wxSOCKET_LOST\n")); break;
	default             : s.Append(_("Unexpected event !\n")); break;
	}

  m_text->AppendText(s);

switch(event.GetSocketEvent()){
case wxSOCKET_INPUT:
	{
	sock->SetNotify(wxSOCKET_LOST_FLAG);

	char buf[4096];

	wxString sTaille;
	long len2 = sock->Read(buf, sizeof(buf)).LastCount();
	if (len2>0)
		{
		buf[len2]=0;
		GestionRequete(sock,buf);
		}

	break;
	}
case wxSOCKET_LOST:
	{
	m_numClients--;
	wxLogMessage("Deleting socket.");
	sock->Destroy();
	break;
	}
default: ;
	}

 UpdateStatusBar();
}

// convenience functions

void MyFrame::UpdateStatusBar()
{
#if wxUSE_STATUSBAR
  wxString s;
  s.Printf(_("%d clients connected"), m_numClients);
  SetStatusText(s, 1);
#endif // wxUSE_STATUSBAR
}





void MyFrame::GestionRequete(wxSocketBase *sock,char *buf)
{

wxString item(buf);
wxLogMessage("Buffer %s",buf);
wxString		dossierAct=wxGetCwd();
#ifdef __PRESTATAIRE_SCILAB__
ScilabThread     *m_pThread = new ScilabThread(item,1);
m_pThread->sock=(void*)sock;
m_pThread->tpsCreation=wxGetLocalTime();
m_pThread->Entree();
wxString nomFicArg;
nomFicArg.Printf("tmp%d.txt",nbAppel++);
nomFicArg=m_pThread->nomDossierLog+"/"+nomFicArg;
wxFile fArg(nomFicArg,wxFile::write);
fArg.Write(item);
fArg.Close();
wxString process(REQUETE_SCILAB);
process=process+" \""+nomFicArg+"\"";
int pid=wxExecute(process,wxEXEC_ASYNC,wp);
listeRequete[pid]=m_pThread;
#else
ScilabThread     *m_pThread = new ScilabThread(item);
m_pThread->Entree();
#endif

if (listeRequete.size()>0)
	{
	map<int,ScilabThread *>::iterator pp;
	long tpsAct=wxGetLocalTime();
	for (pp=listeRequete.begin();pp!=listeRequete.end();pp++)
		{
		ScilabThread     *m1=pp->second;
		if (tpsAct-m1->tpsCreation>30)
			{
			wxLogMessage("Arret Process %ld",pp->first);
			if (!wxKill  ( pp->first,wxSIGKILL))
				{
				wxLogMessage("Process %ld arreter",pp->first);
				wxLogMessage("Fermeture socket");
				wxSocketBase *sck=(wxSocketBase *)m_pThread->sock;
				sck->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
				listeRequete.erase(pp->first);
				}
			}
		}
	}
}


void MyFrame::FinRequete(wxProcessEvent& event)
{
ScilabThread     *m_pThread=listeRequete[event.GetPid()];
if (m_pThread==NULL)
	{
	wxLogMessage("Unknown Thread");
	return;
	}
wxLogMessage("Fin requete %d %d %d",(int)m_pThread->GetCurrentId(),event.GetPid(),event.GetExitCode());
wxSocketBase *sock=(wxSocketBase *)m_pThread->sock;
sock->SetFlags(wxSOCKET_WAITALL);
if (!sock->IsClosed()  && sock->IsOk() && sock->IsConnected() )
	{
	wxString		dossierAct=wxGetCwd();
	wxSetWorkingDirectory(dossierAct);
	int nbFic=0;
	bool st=false;
	if (m_pThread->variableCGI["uidUser"].Length()>0)
		wxLogMessage("uuid %s ==> Script %s",m_pThread->variableCGI["uidUser"],m_pThread->variableCGI["script"]);
	else
		wxLogMessage("uuid %s ==> Script %s",m_pThread->variableCGI["uidUser"],m_pThread->variableCGI["script"]);
	do
		{
		wxString nomParam;
		nomParam.Printf("nomFic%d",nbFic);
		if (m_pThread->variableCGI[nomParam]!="")
			{
			fstream fSci;

			wxString nom(m_pThread->variableCGI[nomParam]);

			wxString nomFichier=m_pThread->nomDossierResultat+"/"+nom.SubString(1,nom.Length()-2);

	#ifdef _MSC_VER
	nomFichier.Replace("/","\\");
	#endif
			fSci.open(nomFichier.ToAscii(),ios::in|ios::binary);
			if (fSci.is_open())
				{
				fSci.seekg (0 , ios_base::end);
				size_t taille = fSci.tellp ();
				fSci.seekg (0,ios_base::beg);
				char *buffer=new char[taille];
				fSci.read(buffer,taille);
				fSci.close();

				// Nom du fichier
				wxString chaineTaille;
				chaineTaille.Printf("%10ld",m_pThread->variableCGI[nomParam].Length());
				sock->Write(chaineTaille.c_str(),10);
				sock->Write(m_pThread->variableCGI[nomParam],m_pThread->variableCGI[nomParam].Length());
				// Taille du fichier
				chaineTaille.Printf("%10ld",taille);
				sock->Write(chaineTaille.c_str(),10);
				// Le fichier
				long nbOctetEnvoye=0,cumulOctetEnvoye=0,pb=0;
				do {

					sock->Write(buffer+nbOctetEnvoye,taille-nbOctetEnvoye);
					nbOctetEnvoye = (long)sock->LastCount();
					if (nbOctetEnvoye==0)
						{
						pb++;
						wxMilliSleep(500);
						}
					else 
						pb=0;
					cumulOctetEnvoye += nbOctetEnvoye;
					}
				while (pb<2 && cumulOctetEnvoye<taille );
				delete buffer;
				wxLogMessage("File %s: %ld/%ld",m_pThread->variableCGI[nomParam],nbOctetEnvoye,(long)taille);
				st=true;

				nbFic++;
				}
			else
				st=false;
			}
		else
			st=false;
		}
	while (st);
	delete m_pThread;
	listeRequete.erase(event.GetPid());
	sock->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
	}
else
	{
	delete m_pThread;
	listeRequete.erase(event.GetPid());
	sock->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
	}

wxLogMessage("Fin requete");
}
